==> usbwatch-0.1.tar.bz2: usbwatch-0.1/desk.cc <==
#include "desk.h"
#include "icon.h"

Desk::Desk (int *argc, char **argv)
{
  ImlibInitParams ii;
  ImlibImage *img;
  int mj, mn;
  toplevel = XtOpenApplication (&app_context, "UsbWatch", NULL, 0, argc,
				argv, NULL, applicationShellWidgetClass, NULL,
				0);
  if(!XShapeQueryVersion ( XtDisplay (toplevel), &mj, &mn )) {
  fprintf(stderr, "XShape Extention not available\n");
  exit(1);
  }
#ifdef PRNDEBUG
  printf("XShape ver.%d.%d\n", mj, mn);
#endif
  XtConfigureWidget (toplevel, 10, 10, 100, 100, 5);
  //XtRealizeWidget (toplevel);
  ii.imagecachesize = 0;
  ii.pixmapcachesize = 0;
  ii.sharedmem = 0;
  ii.sharedpixmaps = 0;
  ii.flags = PARAMS_IMAGECACHESIZE | PARAMS_PIXMAPCACHESIZE | PARAMS_SHAREDMEM |PARAMS_SHAREDPIXMAPS;
  Data = Imlib_init_with_params (XtDisplay (toplevel), &ii);
  img = Imlib_create_image_from_xpm_data(Data, unknown_xpm);
  if(!img) { fprintf(stderr, "Error loading builtin \"unknown\" xpm\n"); 
              exit(1); }
  UPixmapWidth = img->rgb_width;
  UPixmapHeight = img->rgb_height;
  Imlib_render(Data,img,UPixmapWidth,UPixmapHeight);
  UnknownPixmap = Imlib_move_image(Data,img);
  UnknownMask = Imlib_move_mask(Data,img);
  Imlib_destroy_image(Data, img);
  img = Imlib_create_image_from_xpm_data(Data, empty_xpm);
  if(!img) { fprintf(stderr, "Error loading builtin \"empty\" xpm\n");
             exit(1); }
  EPixmapWidth = img->rgb_width;
  EPixmapHeight = img->rgb_height;
  Imlib_render(Data,img,EPixmapWidth,EPixmapHeight);
  EmptyPixmap  = Imlib_move_image(Data,img);
  EmptyMask = Imlib_move_mask(Data,img);
  Imlib_destroy_image(Data, img);
  LoadOptions ();
#ifdef PRNDEBUG
  printf("Options loaded\n");
#endif
  LoadIcons ();
#ifdef PRNDEBUG
  printf ("Icons Loaded\n");
#endif
  JobTime = 0;
  //if(Jobs.size() > 0) { 
  //XtAppAddWorkProc(app_context, (XtWorkProc)procExecJobs, 
  //                 (XtPointer) this);
  //signal(SIG_ALM, 
  //}

}

Desk::~Desk ()
{
  XFreeFontSet( XtDisplay (toplevel), fontset);
}

void
Desk::LoadOptions (void)
{
  Database *Db;
  Table *Config;
  char **lfonts;
  int lcount;
  string File (getenv ("HOME"));
  
  // Get our Database.
  File = "/etc/X11/usbwatch/usbwatchrc";
  if( access( File.c_str(), R_OK ) ) {
    fprintf(stderr, "Can't access %s\n", File.c_str() );
    File = getenv ("HOME");
    File += "/.usbwatchrc";
    if( access( File.c_str(), R_OK ) )
      fprintf(stderr, "Can't access %s\n", File.c_str() );
    }
  Db = new Database (File);
  if (!(Config = Db->Query ("Config")))
    {
      fprintf
	(stderr, "Can't find config file, or missing 'Config' table in the config file %s.\n", File.c_str() );
      exit (-1);
    }
  delete Db;
  // Start handling the various chunks.
  //
  ClickInterval = atoi ((char *) Config->Query ("ClickInterval").c_str ());
  if (ClickInterval <= 0)
    ClickInterval = 500;
#ifdef PRNDEBUG
  printf ("ClickInterval - %d\n", ClickInterval);
#endif
// font
{
 char **missing_list;
 char *def_string;
 int cscount;
 fontset =  XCreateFontSet(XtDisplay (toplevel), 
        (char *) Config->Query ("FontName").c_str (),
//        "-misc-*-*-*-*--*-*-*-*-*-*-iso10646-1"
        &missing_list, &cscount, &def_string
         );
 if(!fontset) { fprintf(stderr, "Unable create FontSet for FontName: \"%s\"\n", 
                (char *) Config->Query ("FontName").c_str ()
              ); 
    exit(1);
    }
#ifdef PRNDEBUG
    else
   { printf("fontset created\n"); }
   for( int q=0; q<cscount; q++ )
      printf("Missing charset: %s\n", missing_list[q] );
      printf("Default string: %s\n", def_string);
#endif
}
  //color
  if (!XAllocNamedColor (XtDisplay (toplevel),
			 DefaultColormap (XtDisplay (toplevel),
					  DefaultScreen (XtDisplay
							 (toplevel))),
			 (char *) Config->Query ("ShadowColor").c_str (),
			 &FontBack, &FontBack))
    {
      fprintf (stderr, "Cannot allocate color %s\n",
	      (char *) Config->Query ("ShadowColor").c_str ());
      XAllocNamedColor (XtDisplay (toplevel),
			DefaultColormap (XtDisplay (toplevel),
		        DefaultScreen (XtDisplay (toplevel))), "black",
			&FontFore, &FontBack);
    }
  if (!XAllocNamedColor (XtDisplay (toplevel),
			 DefaultColormap (XtDisplay (toplevel),
					  DefaultScreen (XtDisplay
							 (toplevel))),
			 (char *) Config->Query ("FontColor").c_str (),
			 &FontFore, &FontFore))
    {
      fprintf (stderr, "Cannot allocate color %s\n",
	      (char *) Config->Query ("ShadowColor").c_str ());
      XAllocNamedColor (XtDisplay (toplevel),
			DefaultColormap (XtDisplay (toplevel),
			DefaultScreen (XtDisplay (toplevel))), "white",
			&FontFore, &FontFore);
    }
  CaptionBack = makeBooleanFromStr(
         (char *) Config->Query ("CaptionBackground").c_str (), -1);
  CaptionBorder = makeBooleanFromStr(
         (char *) Config->Query ("CaptionBorder").c_str (), -1);
#ifdef PRNDEBUG
  printf("CaptionBackground: %d, CaptionBorder:%d\n", CaptionBack, CaptionBorder);
#endif
  snapX = atoi( (char *) Config->Query ("SnapWidth").c_str () );
  snapY = atoi( (char *) Config->Query ("SnapHeight").c_str () );
  snapX = (snapX>0) ? snapX : 1;
  snapY = (snapY>0) ? snapY : 1;
  IconWidth = atoi( (char *) Config->Query ("IconWidth").c_str () );
  IconHeight = atoi( (char *) Config->Query ("IconHeight").c_str () );
  char *fixed = (char *) Config->Query ("Fixed").c_str ();
  if( !fixed || strlen(fixed) == 0 ) IconFixed = -1;
    else
  IconFixed = ( !strcasecmp(fixed, "yes") || !strcasecmp(fixed, "true") ||
                !strcasecmp(fixed, "on")  || !strcasecmp(fixed, "1")  ||
                !strcasecmp(fixed, "y") ) ? 1 : 0;
  //printf("IconWidth - %d, IconHeight - %d, IconFixed - %d \n", IconWidth, IconHeight, IconFixed);
  delete Config;
}
static int Filter_Links (const struct dirent * dir)
{
 char *p;
 static char *ext =".lnk";
 static int  extlen = strlen(ext);
 if( !(p = strstr(dir->d_name,  ext)) ) return 0;
 if( strlen(p) != extlen ) return 0;
 printf("%s\n", dir->d_name);
 return 1;
}

void
Desk::LoadIcons ()
{
  struct dirent **Files;
  string FileName, DirName (getenv ("HOME"));
  Database *IconDb;
  Table *IconTb;
  Icon *I;
  int FileCount;

  DirName = "/etc/X11/usbwatch/";
  //FileCount = scandir (DirName.c_str (), &Files, 0, alphasort);
  FileCount = scandir (DirName.c_str (), &Files, &Filter_Links, alphasort);

  for (int i = 0; i < FileCount; i++)
    {
      FileName = DirName + Files[i]->d_name;
      try {
         CreateIcon((char*) FileName.c_str ());
	 }
      catch ( char *errStr ) {
         fprintf(stderr, errStr);
	 exit(1);
      }
    /***
      IconDb = new Database (FileName);
      if (!(IconTb = IconDb->Query ("Icon")))
	{
	  free (Files[i]);
	  delete IconDb;
	  continue;
	}
      try  {
         I = new Icon (this, (char *) FileName.c_str (), IconTb);
         Icons.push_back (I);
	 }
      catch ( char *errStr ) {
      }
      free (Files[i]);
      delete IconDb;
      delete IconTb;
    ***/
    }
  free (Files);
  if (Icons.size () == 0)
    {
      fprintf (stderr, "No icons found\n");
      _exit (1);
    }
}

void
Desk::setCheckMountProc (Icon * I)
{
#ifdef PRNDEBUG
  printf("Setting check proc for %s\n", I->getCaption());
#endif
  struct BackgroundJobStruct *job =
    (struct BackgroundJobStruct *)
    malloc (sizeof (struct BackgroundJobStruct));
  job->procPtr = (XtWorkProc) this->procIsMounted;
  job->xtPtr = (XtPointer) I;
  Jobs.push_back (job);
}

void
Desk::procIsMounted (Icon * I)
{
  I->updateMount (I->getDesk ()->isMounted (I->getCaption ()));
}

Boolean
Desk::isMounted (char *mnt)
{
  int i;
  for (i = 0; i < mounts.size (); i++)
    if (!strcmp(mounts[i],mnt))
      return True;
  return false;
}

Boolean
Desk::procExecJobs (XtPointer D)
{
  return ((Desk *) D)->ExecJobs ();
}

Boolean
Desk::ExecJobs (void)
{
  int i;
  time_t t = time (NULL);
  if (t - JobTime < 1)
    {
      usleep (50000);
      return False;
    }
  JobTime = t;
  MountCheck ();
//  if (!MountCheck ())
//    {
//      usleep (50000);
//      return False;
//    }
//  printf ("Executing %d jobs\n", Jobs.size ());
  for (i = 0; i < Jobs.size (); i++)
    (*Jobs[i]->procPtr) (Jobs[i]->xtPtr);
  return False;
}

Boolean
Desk::MountCheck (void)
{
#ifdef LINUX
  char *p;
  stat (MTABFILE, &buf);
  if (buf.st_mtime == last_time)
    return False;
  last_time = buf.st_mtime;
  struct mntent *mnt;
  for(int i = 0; i<mounts.size(); i++) {
     free(mounts[i]);
  }
  mounts.clear ();
  FILE *f = setmntent (MTABFILE, "r");
  while ((mnt = getmntent (f)))
    {
      p = strdup(mnt->mnt_dir);
      mounts.push_back ( p );
    }
  endmntent (f);
  return True;
#endif
#ifdef FREEBSD
  char *p;
  struct statfs buff[20];
  int i, rc;
  rc = getfsstat( buff, 20*sizeof(struct statfs), MNT_NOWAIT);
  for(int i = 0; i<mounts.size(); i++) free(mounts[i]);
  mounts.clear ();
  for(int i = 0; i<rc; i++ ) {
    p = strdup(buff[i].f_mntonname);
    mounts.push_back ( p );
    }
  return True;
#endif
return False;
}
void Desk::DeleteIcon(Icon *I) {
     int i;
     // delete mountProc
     for (i = 0; i < Jobs.size (); i++ ) {
         if( Jobs[i]->xtPtr == I ) {
	     free(Jobs[i]);
	     Jobs.erase(Jobs.begin()+i);
	     break;
	     }
     }
     // delete from icon list
     for( i=0; i<Icons.size(); i++) {
        if( Icons[i] == I ) {
	  Icons.erase(Icons.begin()+i);
	  break;
	  }
     }
     delete I;    
}
int Desk::CreateIcon(char *cfgFile) {
    Icon *I;
    Database *IconDb;
    Table *IconTb;
    IconDb = new Database (cfgFile);
    if (!(IconTb = IconDb->Query ("Icon"))) {
       delete IconDb;
       return 0;
       }
    I = new Icon (this, cfgFile, IconTb);
    Icons.push_back (I);
    delete IconDb;
    return 1;
}
void Desk::LowerIcons(Icon *I) {
     int i;
     // delete from icon list
     for( i=0; i<Icons.size(); i++) {
        if( Icons[i] != I ) {
          Icons[i]->lowerme();
          }
     }
}

==> usbwatch-0.3.tar.bz2: usbwatch-0.3/desk.cc <==
#include "desk.h"
#include "icon.h"

Desk::Desk (int *argc, char **argv)
{
  ImlibInitParams ii;
  ImlibImage *img;
  int mj, mn;
  toplevel = XtOpenApplication (&app_context, "UsbWatch", NULL, 0, argc,
				argv, NULL, applicationShellWidgetClass, NULL,
				0);
  if(!XShapeQueryVersion ( XtDisplay (toplevel), &mj, &mn )) {
  fprintf(stderr, "XShape Extention not available\n");
  exit(1);
  }
#ifdef PRNDEBUG
  printf("XShape ver.%d.%d\n", mj, mn);
#endif
  XtConfigureWidget (toplevel, 10, 10, 100, 100, 5);
  //XtRealizeWidget (toplevel);
  ii.imagecachesize = 0;
  ii.pixmapcachesize = 0;
  ii.sharedmem = 0;
  ii.sharedpixmaps = 0;
  ii.flags = PARAMS_IMAGECACHESIZE | PARAMS_PIXMAPCACHESIZE | PARAMS_SHAREDMEM |PARAMS_SHAREDPIXMAPS;
  Data = Imlib_init_with_params (XtDisplay (toplevel), &ii);
  img = Imlib_create_image_from_xpm_data(Data, unknown_xpm);
  if(!img) { fprintf(stderr, "Error loading builtin \"unknown\" xpm\n"); 
              exit(1); }
  UPixmapWidth = img->rgb_width;
  UPixmapHeight = img->rgb_height;
  Imlib_render(Data,img,UPixmapWidth,UPixmapHeight);
  UnknownPixmap = Imlib_move_image(Data,img);
  UnknownMask = Imlib_move_mask(Data,img);
  Imlib_destroy_image(Data, img);
  img = Imlib_create_image_from_xpm_data(Data, empty_xpm);
  if(!img) { fprintf(stderr, "Error loading builtin \"empty\" xpm\n");
             exit(1); }
  EPixmapWidth = img->rgb_width;
  EPixmapHeight = img->rgb_height;
  Imlib_render(Data,img,EPixmapWidth,EPixmapHeight);
  EmptyPixmap  = Imlib_move_image(Data,img);
  EmptyMask = Imlib_move_mask(Data,img);
  Imlib_destroy_image(Data, img);
  LoadOptions ();
#ifdef PRNDEBUG
  printf("Options loaded\n");
#endif
  LoadIcons ();
#ifdef PRNDEBUG
  printf ("Icons Loaded\n");
#endif
  JobTime = 0;
  //if(Jobs.size() > 0) { 
  //XtAppAddWorkProc(app_context, (XtWorkProc)procExecJobs, 
  //                 (XtPointer) this);
  //signal(SIG_ALM, 
  //}

}

Desk::~Desk ()
{
  XFreeFontSet( XtDisplay (toplevel), fontset);
}

void
Desk::LoadOptions (void)
{
  Database *Db;
  Table *Config;
  char **lfonts;
  int lcount;
  string File (getenv ("HOME"));
  
  // Get our Database.
  File += "/.usbwatch/usbwatchrc";
  if( access( File.c_str(), R_OK ) ) {
    fprintf(stderr, "Can't access %s\n", File.c_str() );
    File = getenv ("HOME");
    File += "/.usbwatchrc";
    if( access( File.c_str(), R_OK ) )
      fprintf(stderr, "Can't access %s\n", File.c_str() );
    }
  Db = new Database (File);
  if (!(Config = Db->Query ("Config")))
    {
      fprintf
	(stderr, "Can't find config file, or missing 'Config' table in the config file %s.\n", File.c_str() );
      exit (-1);
    }
  delete Db;
  // Start handling the various chunks.
  //
  ClickInterval = atoi ((char *) Config->Query ("ClickInterval").c_str ());
  if (ClickInterval <= 0)
    ClickInterval = 500;
#ifdef PRNDEBUG
  printf ("ClickInterval - %d\n", ClickInterval);
#endif
// font
{
 char **missing_list;
 char *def_string;
 int cscount;
 fontset =  XCreateFontSet(XtDisplay (toplevel), 
        (char *) Config->Query ("FontName").c_str (),
//        "-misc-*-*-*-*--*-*-*-*-*-*-iso10646-1"
        &missing_list, &cscount, &def_string
         );
 if(!fontset) { fprintf(stderr, "Unable create FontSet for FontName: \"%s\"\n", 
                (char *) Config->Query ("FontName").c_str ()
              ); 
    exit(1);
    }
#ifdef PRNDEBUG
    else
   { printf("fontset created\n"); }
   for( int q=0; q<cscount; q++ )
      printf("Missing charset: %s\n", missing_list[q] );
      printf("Default string: %s\n", def_string);
#endif
}
  //color
  if (!XAllocNamedColor (XtDisplay (toplevel),
			 DefaultColormap (XtDisplay (toplevel),
					  DefaultScreen (XtDisplay
							 (toplevel))),
			 (char *) Config->Query ("ShadowColor").c_str (),
			 &FontBack, &FontBack))
    {
      fprintf (stderr, "Cannot allocate color %s\n",
	      (char *) Config->Query ("ShadowColor").c_str ());
      XAllocNamedColor (XtDisplay (toplevel),
			DefaultColormap (XtDisplay (toplevel),
		        DefaultScreen (XtDisplay (toplevel))), "black",
			&FontFore, &FontBack);
    }
  if (!XAllocNamedColor (XtDisplay (toplevel),
			 DefaultColormap (XtDisplay (toplevel),
					  DefaultScreen (XtDisplay
							 (toplevel))),
			 (char *) Config->Query ("FontColor").c_str (),
			 &FontFore, &FontFore))
    {
      fprintf (stderr, "Cannot allocate color %s\n",
	      (char *) Config->Query ("ShadowColor").c_str ());
      XAllocNamedColor (XtDisplay (toplevel),
			DefaultColormap (XtDisplay (toplevel),
			DefaultScreen (XtDisplay (toplevel))), "white",
			&FontFore, &FontFore);
    }
  CaptionBack = makeBooleanFromStr(
         (char *) Config->Query ("CaptionBackground").c_str (), -1);
  CaptionBorder = makeBooleanFromStr(
         (char *) Config->Query ("CaptionBorder").c_str (), -1);
#ifdef PRNDEBUG
  printf("CaptionBackground: %d, CaptionBorder:%d\n", CaptionBack, CaptionBorder);
#endif
  snapX = atoi( (char *) Config->Query ("SnapWidth").c_str () );
  snapY = atoi( (char *) Config->Query ("SnapHeight").c_str () );
  snapX = (snapX>0) ? snapX : 1;
  snapY = (snapY>0) ? snapY : 1;
  IconWidth = atoi( (char *) Config->Query ("IconWidth").c_str () );
  IconHeight = atoi( (char *) Config->Query ("IconHeight").c_str () );
  char *fixed = (char *) Config->Query ("Fixed").c_str ();
  if( !fixed || strlen(fixed) == 0 ) IconFixed = -1;
    else
  IconFixed = ( !strcasecmp(fixed, "yes") || !strcasecmp(fixed, "true") ||
                !strcasecmp(fixed, "on")  || !strcasecmp(fixed, "1")  ||
                !strcasecmp(fixed, "y") ) ? 1 : 0;
  //printf("IconWidth - %d, IconHeight - %d, IconFixed - %d \n", IconWidth, IconHeight, IconFixed);
  delete Config;
}
static int Filter_Links (const struct dirent * dir)
{
 char *p;
 static char *ext =".lnk";
 static int  extlen = strlen(ext);
 if( !(p = strstr(dir->d_name,  ext)) ) return 0;
 if( strlen(p) != extlen ) return 0;
 printf("%s\n", dir->d_name);
 return 1;
}

void
Desk::LoadIcons ()
{
  struct dirent **Files;
  string FileName, DirName (getenv ("HOME"));
  Database *IconDb;
  Table *IconTb;
  Icon *I;
  int FileCount;

  DirName += "/.usbwatch/icons/";
  //FileCount = scandir (DirName.c_str (), &Files, 0, alphasort);
  FileCount = scandir (DirName.c_str (), &Files, &Filter_Links, alphasort);

  for (int i = 0; i < FileCount; i++)
    {
      FileName = DirName + Files[i]->d_name;
      try {
         CreateIcon((char*) FileName.c_str ());
	 }
      catch ( char *errStr ) {
         fprintf(stderr, errStr);
	 exit(1);
      }
    /***
      IconDb = new Database (FileName);
      if (!(IconTb = IconDb->Query ("Icon")))
	{
	  free (Files[i]);
	  delete IconDb;
	  continue;
	}
      try  {
         I = new Icon (this, (char *) FileName.c_str (), IconTb);
         Icons.push_back (I);
	 }
      catch ( char *errStr ) {
      }
      free (Files[i]);
      delete IconDb;
      delete IconTb;
    ***/
    }
  free (Files);
  if (Icons.size () == 0)
    {
      fprintf (stderr, "No icons found\n");
      _exit (1);
    }
}

void
Desk::setCheckMountProc (Icon * I)
{
#ifdef PRNDEBUG
  printf("Setting check proc for %s\n", I->getCaption());
#endif
  struct BackgroundJobStruct *job =
    (struct BackgroundJobStruct *)
    malloc (sizeof (struct BackgroundJobStruct));
  job->procPtr = (XtWorkProc) this->procIsMounted;
  job->xtPtr = (XtPointer) I;
  Jobs.push_back (job);
}

void
Desk::procIsMounted (Icon * I)
{
  I->updateMount (I->getDesk ()->isMounted (I->getCaption ()));
}

Boolean
Desk::isMounted (char *mnt)
{
  int i;
  for (i = 0; i < mounts.size (); i++)
    if (!strcmp(mounts[i],mnt))
      return True;
  return false;
}

Boolean
Desk::procExecJobs (XtPointer D)
{
  return ((Desk *) D)->ExecJobs ();
}

Boolean
Desk::ExecJobs (void)
{
  int i;
  time_t t = time (NULL);
  if (t - JobTime < 1)
    {
      usleep (50000);
      return False;
    }
  JobTime = t;
  MountCheck ();
//  if (!MountCheck ())
//    {
//      usleep (50000);
//      return False;
//    }
//  printf ("Executing %d jobs\n", Jobs.size ());
  for (i = 0; i < Jobs.size (); i++)
    (*Jobs[i]->procPtr) (Jobs[i]->xtPtr);
  return False;
}

Boolean
Desk::MountCheck (void)
{
#ifdef LINUX
  char *p;
  stat (MTABFILE, &buf);
  if (buf.st_mtime == last_time)
    return False;
  last_time = buf.st_mtime;
  struct mntent *mnt;
  for(int i = 0; i<mounts.size(); i++) {
     free(mounts[i]);
  }
  mounts.clear ();
  FILE *f = setmntent (MTABFILE, "r");
  while ((mnt = getmntent (f)))
    {
      p = strdup(mnt->mnt_dir);
      mounts.push_back ( p );
    }
  endmntent (f);
  return True;
#endif
#ifdef FREEBSD
  char *p;
  struct statfs buff[20];
  int i, rc;
  rc = getfsstat( buff, 20*sizeof(struct statfs), MNT_NOWAIT);
  for(int i = 0; i<mounts.size(); i++) free(mounts[i]);
  mounts.clear ();
  for(int i = 0; i<rc; i++ ) {
    p = strdup(buff[i].f_mntonname);
    mounts.push_back ( p );
    }
  return True;
#endif
return False;
}
void Desk::DeleteIcon(Icon *I) {
     int i;
     // delete mountProc
     for (i = 0; i < Jobs.size (); i++ ) {
         if( Jobs[i]->xtPtr == I ) {
	     free(Jobs[i]);
	     Jobs.erase(Jobs.begin()+i);
	     break;
	     }
     }
     // delete from icon list
     for( i=0; i<Icons.size(); i++) {
        if( Icons[i] == I ) {
	  Icons.erase(Icons.begin()+i);
	  break;
	  }
     }
     delete I;    
}
int Desk::CreateIcon(char *cfgFile) {
    Icon *I;
    Database *IconDb;
    Table *IconTb;
    IconDb = new Database (cfgFile);
    if (!(IconTb = IconDb->Query ("Icon"))) {
       delete IconDb;
       return 0;
       }
    I = new Icon (this, cfgFile, IconTb);
    Icons.push_back (I);
    delete IconDb;
    return 1;
}
void Desk::LowerIcons(Icon *I) {
     int i;
     // delete from icon list
     for( i=0; i<Icons.size(); i++) {
        if( Icons[i] != I ) {
          Icons[i]->lowerme();
          }
     }
}
